#****
# [manpage_begin fetch n 0.5]
# [moddesc {Mail fetching for Mailroom}]
# [copyright {Copyright 2009 Jeremy Cowgar. All Rights Reserved.}]
# [category {Mailroom Fetching}]
# [require pop3 1.6.3]
# [require tls]
# [keywords mailroom pop3 fetch]
# [description]
# Provides methods to fetch mail from various sources.
#

namespace eval ::mailroom::fetch {
  package require pop3 1.6.3
  
  #****
  # [section Commands]
  #
  # [list_begin definitions]
  # 
  # [call [cmd {::mailroom::fetch begin}] [opt [arg statusCmd]]]
  #
  # Begin a fetch from the configured mail servers. If [arg statusCmd] is non-zero, it will be called
  # during key points to update the status of the fetch process.
  #
  proc begin {{statusCmd 0}} {
    array set pop3cfg [::mailroom::db eval {
        SELECT name2, value FROM var WHERE name1='config' AND name2 LIKE 'pop3%'
      }]
    
    set cmd [list ::pop3::open]
    if {$pop3cfg(pop3tls)} {
      package require tls
      lappend cmd {-socketcmd ::tls::socket}
    }
    lappend cmd $pop3cfg(pop3host)
    lappend cmd $pop3cfg(pop3userid)
    lappend cmd $pop3cfg(pop3passwd)
    lappend cmd $pop3cfg(pop3port)
    
    if {$statusCmd !=0} {
      $statusCmd "Connecting"
    }
    
    log::log notice "Fetching email from $pop3cfg(pop3host) user $pop3cfg(pop3userid)"
    
    set cmd [join $cmd]
    set server [eval $cmd]
    
    if {$statusCmd != 0} {
      $statusCmd {Connected, fetching...}
    }
    
    set status [split [::pop3::status $server]]
    set msgCount [lindex $status 0]
    set gotCount 0
    for {set x 1} {$x <= [lindex $status 0]} {incr x} {
      if {$statusCmd != 0} {
        $statusCmd "Fetching $x of $msgCount"
      }
      set content [::pop3::retrieve $server $x]
      ::mailroom::message insert -string [lindex $content 0]
      if {$pop3cfg(pop3delete)} {
        ::pop3::delete $server $x
      }
      incr gotCount
    }
    
    ::pop3::close $server
    
    log::log notice "Fetch done, $gotCount new messages."
    if {$statusCmd != 0} {
      $statusCmd "Last ($gotCount) [clock format [clock seconds] -format {%a, %I:%M%P}]"
    }
  }
  
  #****
  # [list_end]
  
  #****
  # [section Examples]
  # [example {::mailroom::fetch begin}]
  #
  # [section {Todo}]
  # [list_begin enumerated]
  # [enum] No error checking is in place.
  # [list_end]
  #
  # [section {Bugs, Ideas, Feedback}]
  # Provide feedback to Jeremy Cowgar <jeremy@cowgar.com>
  
  namespace export begin
  namespace ensemble create
}

#****
# [manpage_end]
